#!/usr/bin/env python3

import asyncio, sys
from distutils.util import strtobool
import datetime
from asyncio.subprocess import Process
from pathlib import Path

from git_pp.git_push_to_all_remotes import PathLike, git_push_to_all_remote


async def run_command(*args,
                      cwd: PathLike | None = None,
                      silent: bool = False) -> int:
    """Run a command in a subprocess and return the output.

    Args:
        *args: command and arguments

    Returns:
        str: output of the command
    """
    process: Process = await asyncio.create_subprocess_exec(
        *args,
        cwd=cwd,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)
    stdout, _ = await process.communicate()
    if not silent:
        print(stdout.decode())

    rc = await process.wait()
    return rc


def get_iso8601_timestamp():
    """get current time in this format "%Y-%m-%dT%H-%M-%SZ"
    """
    # return datetime.datetime.utcnow().isoformat() + 'Z'
    # return datetime.datetime.now().isoformat()
    now = datetime.datetime.now()
    return now.strftime('%Y-%m-%dT%H-%M-%SZ')


async def git_pre_pull(cwd: PathLike | None = None,
                       commit_message: str | None = None,
                       status_only: bool = False) -> None:
    """
    Args:
        cwd (PathLike | None): working directory
    """
    print(f'📦 Pre-pulling {cwd or "."} ...')
    # https://unix.stackexchange.com/questions/46814/watch-command-not-showing-colors-for-git-status
    await run_command('git', '-c', 'color.status=always', 'status', cwd=cwd)
    await run_command('git', 'add', '--all', cwd=cwd, silent=status_only)
    await run_command(
        'git',
        'commit',
        '-m',
        f'{get_iso8601_timestamp() if not commit_message else commit_message}',
        cwd=cwd,
        silent=status_only)


async def git_pre_pull_and_push_to_all_remote(
        commit_message: str
    | None = None,
        status_only: bool = False,
        remotes: list[str]
    | None = None,
        branch: str | None = None,
        force=False,
        timeout=None,
        cwd: PathLike | None = None) -> tuple[int, PathLike | None]:
    """
    Args:
        force (bool): force push
        timeout (int): timeout in seconds
        cwd (PathLike | None): working directory
    """
    await git_pre_pull(commit_message=commit_message,
                       status_only=status_only,
                       cwd=cwd)
    rc, _, _ = await git_push_to_all_remote(remotes=remotes,
                                            branch=branch,
                                            force=force,
                                            timeout=timeout,
                                            cwd=cwd)
    return rc, cwd


async def git_pre_pull_and_push_to_all_remote_C(dirs: list[PathLike],
                                                commit_message: str
                                                | None = None,
                                                status_only: bool = False,
                                                remotes: list[str]
                                                | None = None,
                                                branch: str | None = None,
                                                force=False,
                                                timeout=None) -> int:
    """
    Args:
        force (bool): force push
        timeout (int): timeout in seconds
        cwd (PathLike | None): working directory
    """
    coros = [
        git_pre_pull_and_push_to_all_remote(commit_message=commit_message,
                                            status_only=status_only,
                                            force=force,
                                            remotes=remotes,
                                            branch=branch,
                                            timeout=timeout,
                                            cwd=cwd) for cwd in dirs
    ]
    # status_codes = asyncio.gather(*ts)
    status_codes = []
    for td in asyncio.as_completed(coros):
        # from icecream import ic
        # ic(td.cr_frame.f_locals)
        # ic(await td.get_coro())
        status_code, cwd = await td
        if status_code == 0:
            print(f'✓ Pre-pulled and pushed to all remotes of {cwd}.')
        else:
            print(f'𐄂 Failed to pre-pull and push to all_remotes of {cwd}.')
        status_codes.append(status_code)
    dirs_s = list(map(str, dirs))
    if not any(status_codes):
        print(
            f'✅ Pushed {"current branch" if branch is None else branch} of {dirs} to all of their remotes successfully.'
            + ('(FORCE)' if force else ''))
        return 0
    else:
        print(
            f'❌ Failed to push {"current branch" if branch is None else branch} of {dirs} to all of their remotes.'
            + ('(FORCE)' if force else ''))
        return 1
    #     # all 0
    #     print(
    #         f'✅ Pre-pulled and pushed {dirs_s} to all of their remotes successfully.'
    #         + ('(FORCE)' if force else ''))
    #     return 0
    # else:
    #     print(
    #         f'❌ Failed to pre-pull and push {dirs_s} to all of their remotes.'
    #         + ('(FORCE)' if force else ''))
    #     return 1


async def main() -> None:
    if len(sys.argv) > 1:
        await git_pre_pull(
            cwd=sys.argv[1],  # type: ignore
            commit_message=sys.argv[2] if len(sys.argv) > 2 else None,
            status_only=bool(strtobool(sys.argv[3]))
            if len(sys.argv) > 3 else False)
    else:
        await git_pre_pull()


if __name__ == '__main__':
    asyncio.run(main())


#!/usr/bin/env python3

import asyncio, sys
from asyncio.subprocess import Process
from pathlib import Path
from typing import AnyStr

PathLike = AnyStr | Path

from git_pp import logger


# logger.add(sys.stdout,
#            colorize=True,
#            format="{level} <green>{time}</green> <level>{message}</level>")
async def git_get_all_remotes(cwd: PathLike | None = None) -> list[str]:
    program = ['git', 'remote']
    proc: Process = await asyncio.create_subprocess_exec(
        *program, stdout=asyncio.subprocess.PIPE, cwd=cwd)
    logger.info(f'Getting all remotes of {cwd} .')
    logger.info(f'Running: {program}')
    # logger.info(f'Args: {proc.__dict__}')  # type: ignore
    stdout, _ = await proc.communicate()
    return stdout.decode().strip().splitlines()


async def git_get_current_branch(cwd: PathLike | None = None) -> str:
    program = ['git', 'branch', '--show-current']
    proc: Process = await asyncio.create_subprocess_exec(
        *program, stdout=asyncio.subprocess.PIPE, cwd=cwd)
    stdout, _ = await proc.communicate()
    return stdout.decode().strip()


# async def git_push_to_remote(branch: str,
#                              remote: str,
#                              force=False,
#                              timeout=None,
#                              cwd: PathLike | None = None) -> tuple[int, str]:
#     """pushes branch to remote named remote, and returns the return code

#     Args:
#         remote (str): name of the remote

#     Returns:
#         int: return code
#     """
#     program = ['git', 'push', remote, branch]
#     if force:
#         program.append('--force')
#     process: Process = await asyncio.create_subprocess_exec(*program, cwd=cwd)
#     print(f'Pushing {branch} to {remote}.')
#     try:
#         status_code = await asyncio.wait_for(process.wait(), timeout=timeout)
#         # print(status_code)
#     except asyncio.TimeoutError:
#         print('Timed out waiting to finish, terminating...')
#         process.terminate()
#         status_code = await process.wait()
#         # print(status_code)
#     return (status_code, remote)


async def git_push_to_remote(remote: str,
                             branch: str | None = None,
                             force=False,
                             timeout=None,
                             cwd: PathLike | None = None) -> tuple[int, str]:
    """pushes branch to remote named remote, and returns the return code

    Args:
        remote (str): name of the remote
        branch (str | None): name of the branch to push

    Returns:
        int: return code
    """
    if branch is None:
        branch = await git_get_current_branch(cwd=cwd)
    program = ['git', 'push', remote, branch]
    logger.info(f'Pushing {branch} to {remote} .')
    logger.info(f'Running: {program}')
    if force:
        program.append('--force')
    process: Process = await asyncio.create_subprocess_exec(*program, cwd=cwd)
    # logger.info(f'Args: {process.__dict__}')  # type: ignore
    print(f'Pushing {branch} to {remote}.')
    try:
        status_code = await asyncio.wait_for(process.wait(), timeout=timeout)
        # print(status_code)
    except asyncio.TimeoutError:
        print('Timed out waiting to finish, terminating...')
        process.terminate()
        status_code = await process.wait()
        # print(status_code)
    return (status_code, remote)


async def git_push_to_all_remote(
        remotes: list[str] | None = None,
        branch: str | None = None,
        force=False,
        timeout=None,
        cwd: PathLike | None = None) -> tuple[int, str, PathLike | None]:

    if remotes is None:
        remotes = await git_get_all_remotes(cwd=cwd)
        PUSH_TO_ALL = True
    else:
        PUSH_TO_ALL = False

    if branch is None:
        branch = await git_get_current_branch(cwd=cwd)

    coros = [
        git_push_to_remote(remote=remote,
                           branch=branch,
                           force=force,
                           timeout=timeout,
                           cwd=cwd) for remote in remotes
    ]
    # status_codes = asyncio.gather(*ts)
    status_codes = []
    for td in asyncio.as_completed(coros):
        # from icecream import ic
        # ic(td.cr_frame.f_locals)
        # ic(await td.get_coro())
        status_code, remote = await td
        if status_code == 0:
            print(f'✓ Pushed {branch} to {remote}.')
        else:
            print(f'𐄂 Failed to push {branch} to {remote}.')
        status_codes.append(status_code)
    if not any(status_codes):
        # all 0
        print(
            f'✅ Pushed {branch} to {"all remotes" if PUSH_TO_ALL else remotes} successfully.'
            + ('(FORCE)' if force else ''))
        return (0, branch, cwd)
    else:
        print(
            f'❌ Failed to push {branch} to some of the remotes in {remotes}.' +
            ('(FORCE)' if force else ''))
        return (1, branch, cwd)


async def git_push_to_all_remote_C(dirs: list[PathLike],
                                   remotes: list[str] | None = None,
                                   branch: str | None = None,
                                   force: bool = False,
                                   timeout: float | None = None) -> int:
    coros = [
        git_push_to_all_remote(remotes=remotes,
                               branch=branch,
                               force=force,
                               timeout=timeout,
                               cwd=cwd) for cwd in dirs
    ]
    # status_codes = asyncio.gather(*ts)
    status_codes = []
    for td in asyncio.as_completed(coros):
        # from icecream import ic
        # ic(td.cr_frame.f_locals)
        # ic(await td.get_coro())
        status_code, curr_branch, cwd = await td
        if status_code == 0:
            print(f'✓ Pushed {curr_branch} to all remotes of {cwd}.')
        else:
            print(f'𐄂 Failed to push {curr_branch} to all_remotes of {cwd}.')
        status_codes.append(status_code)
    if not any(status_codes):
        # all 0
        print(
            f'✅ Pushed {"current branch" if branch is None else branch} of {dirs} to all of their remotes successfully.'
            + ('(FORCE)' if force else ''))
        return 0
    else:
        print(
            f'❌ Failed to push {"current branch" if branch is None else branch} of {dirs} to all of their remotes.'
            + ('(FORCE)' if force else ''))
        return 1


async def main() -> None:
    if len(sys.argv) > 1:
        dirs: list[PathLike] = sys.argv[1:]  # type: ignore
        await git_push_to_all_remote_C(dirs, force=False, timeout=None)
    else:
        await git_push_to_all_remote(force=False, timeout=None)


if __name__ == '__main__':
    asyncio.run(main())


#!/usr/bin/env python3

from pathlib import Path
import typer, asyncio
from .git_push_to_all_remotes import PathLike
# from subprocess import CompletedProcess
from asyncio.subprocess import Process

# cSpell:disable
# https://github.com/faif/python-patterns
# https://github.com/faif/python-patterns.git
# git@github.com:faif/python-patterns.git
# gh repo clone faif/python-patterns
# cSpell:enable

app = typer.Typer(name='clone')


@app.command()
async def clone(git_repo_url: str,
                timeout: float | None = None,
                dest: PathLike = '.',
                name: str | None = None) -> tuple[int, tuple[str, str]]:
    """Clone a git repo to a local directory."""
    if name is None:
        name = git_repo_url.split('/')[-1].split('.')[0]
    dest = Path(dest)
    if not dest.exists():
        dest.mkdir()
    dest = dest / name
    program = ['git', 'clone', git_repo_url, str(dest)]
    process: Process = await asyncio.create_subprocess_exec(*program)
    print(f'Pushing {git_repo_url} to {str(dest)}.')
    try:
        status_code = await asyncio.wait_for(process.wait(), timeout=timeout)
        # print(status_code)
    except asyncio.TimeoutError:
        print('Timed out waiting to finish, terminating...')
        process.terminate()
        status_code = await process.wait()
        # print(status_code)
    return (status_code, (git_repo_url, str(dest)))


if __name__ == '__main__':
    app()


__version__ = '1.10.1'
__app_name__ = 'git-pp'

try:
    from logging_utils_tddschn import get_logger  # type: ignore
    logger, _ = get_logger(__app_name__)
except:
    import logging
    from logging import NullHandler
    logger = logging.getLogger(__name__)
    logger.addHandler(NullHandler())



#!/usr/bin/env python3
"""
Author : Xinyuan Chen <45612704+tddschn@users.noreply.github.com>
Date   : 2022-04-12
"""

import argparse, asyncio
from pathlib import Path
import sys

from . import __version__, __app_name__
from .git_pre_pull import git_pre_pull, git_pre_pull_and_push_to_all_remote_C
from .git_push_to_all_remotes import git_push_to_all_remote_C
import typer

try:
    import uvloop
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
except ImportError:
    pass

app = typer.Typer(name='git-pp', invoke_without_command=True)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{__app_name__} v{__version__}")
        raise typer.Exit()


@app.callback()
async def git_pp(version: bool = typer.Option(
    None,
    "--version",
    "-v",
    help="Show the application's version and exit.",
    callback=_version_callback,
    is_eager=True,
),
                 dirs: list[Path] = typer.Argument(None,
                                                   help='Dirs to operate on'),
                 commit_message: str = typer.Option(None,
                                                    '-m',
                                                    '--commit-message',
                                                    help='commit message'),
                 status_only: bool = typer.Option(False,
                                                  '-so',
                                                  '--status-only',
                                                  help='Prints status only'),
                 push_only: bool = typer.Option(
                     False,
                     '-po',
                     '--push-only',
                     help='Push to all remotes, without pre_pull'),
                 push: bool = typer.Option(False,
                                           '-p',
                                           '--push',
                                           help='Push to all remotes'),
                 remotes: list[str] = typer.Option(None,
                                                   '-r',
                                                   '--remote',
                                                   help='Remote name'),
                 branch: str = typer.Option(None, help='Branch name'),
                 force: bool = typer.Option(False, help='Force push'),
                 timeout: float = typer.Option(
                     None, help='Timeout for a single push')):
    if push and push_only:
        typer.secho('Error: -po and -p are mutually exclusive',
                    fg=typer.colors.RED)
        raise typer.Exit()
    elif push:
        await git_pre_pull_and_push_to_all_remote_C(
            dirs=dirs,
            commit_message=commit_message,
            status_only=status_only,
            remotes=remotes,
            branch=branch,
            force=force,
            timeout=timeout)
    elif push_only:
        await git_push_to_all_remote_C(dirs=dirs,
                                       remotes=remotes,
                                       branch=branch,
                                       force=force,
                                       timeout=timeout)
    else:
        await git_pre_pull(dirs[0] if dirs else '.',
                           commit_message=commit_message,
                           status_only=status_only)


# --------------------------------------------------
def get_args():
    """Get command-line arguments"""

    parser = argparse.ArgumentParser(
        prog=__app_name__,
        description='Git utility for auto-committing and concurrent pushing',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('dirs',
                        metavar='DIRS',
                        nargs='*',
                        help='Dirs to operate on',
                        type=Path,
                        default=['.'])

    parser.add_argument('-m',
                        '--commit-message',
                        help='commit message',
                        metavar='COMMIT_MESSAGE',
                        type=str,
                        default=None)

    # parser.add_argument('-i',
    #                     '--int',
    #                     help='A named integer argument',
    #                     metavar='int',
    #                     type=int,
    #                     default=0)

    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version=f'%(prog)s {__version__}')

    parser.add_argument('-so',
                        '--status-only',
                        help='Prints status only',
                        action='store_true')

    parser.add_argument('-p',
                        '--push',
                        help='Push to all remotes',
                        action='store_true')

    parser.add_argument('-po',
                        '--push-only',
                        help='Push to all remotes, without pre_pull',
                        action='store_true')

    parser.add_argument('-r',
                        '--remote',
                        help='Remote name',
                        metavar='REMOTE',
                        type=str,
                        default=None,
                        nargs='+')

    parser.add_argument('-b',
                        '--branch',
                        help='Branch name',
                        metavar='BRANCH',
                        type=str,
                        default=None)

    parser.add_argument('-f',
                        '--force',
                        help='Force push',
                        action='store_true')

    parser.add_argument('-t',
                        '--timeout',
                        help='Timeout for a single push',
                        metavar='TIMEOUT',
                        type=float,
                        default=None)

    return parser.parse_args()


async def main():
    args = get_args()
    dirs = args.dirs
    commit_message = args.commit_message
    status_only = args.status_only
    push_only = args.push_only
    push = args.push
    remotes = args.remote
    branch = args.branch
    force = args.force
    timeout = args.timeout

    if push and push_only:
        sys.exit('Error: -po and -p are mutually exclusive')
    elif push:
        await git_pre_pull_and_push_to_all_remote_C(
            dirs=dirs,
            commit_message=commit_message,
            status_only=status_only,
            remotes=remotes,
            branch=branch,
            force=force,
            timeout=timeout)
    elif push_only:
        await git_push_to_all_remote_C(dirs=dirs,
                                       remotes=remotes,
                                       branch=branch,
                                       force=force,
                                       timeout=timeout)
    else:
        await git_pre_pull(dirs[0] if dirs else '.',
                           commit_message=commit_message,
                           status_only=status_only)


def main_sync():
    # asyncio.run(main())
    asyncio.run(app())


if __name__ == '__main__':
    main_sync()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['git_pp']

package_data = \
{'': ['*']}

install_requires = \
['typer>=0.4.1,<0.5.0', 'uvloop[uvloop]>=0.16.0,<0.17.0']

entry_points = \
{'console_scripts': ['git-pp = git_pp.git_pp:main_sync']}

setup_kwargs = {
    'name': 'git-pp',
    'version': '1.10.1',
    'description': 'A (tiny) Git utility for auto-committing and concurrent pushing',
    'long_description': '# git pp\n\nA (tiny) Git utility for auto-committing and concurrent pushing.\n\nPowered by `asyncio`, with no dependency besides `python>=3.10` and `git`.\n\n- [git pp](#git-pp)\n  - [Features](#features)\n  - [Demo](#demo)\n  - [Use cases and example usage](#use-cases-and-example-usage)\n  - [Installation](#installation)\n    - [pipx](#pipx)\n    - [pip](#pip)\n    - [AUR](#aur)\n  - [Usage](#usage)\n  - [Develop](#develop)\n## Features\n- Auto-stages and commits with custom or generated commit messages\n- Pushes to multiple or all remotes of a git repository **concurrently** with `asyncio`\n- Operates on **any number of git repositories** at the same time\n\n## Demo\n\n<!-- [![asciicast](https://asciinema.org/a/487579.png)](https://asciinema.org/a/487579) -->\n<!-- <a href="https://asciinema.org/a/487579"><img src="https://asciinema.org/a/487579.png" alt="asciicast" style="width:500px;height:300px;"></a> -->\n<a href="https://asciinema.org/a/487579"><img src="https://asciinema.org/a/487579.svg" alt="Asciicast" width="650"/></a>\n\nIn this demo, git pp did the following in \\~/config and \\~/gui repos:\n\n- (Concurrently) Auto staged all changes and commits with ISO-8601 timestamps as commit messages;\n- (Concurrently) Pushed the changes in the checked out branch to all of their remotes, in this case, they’re origin and lab.\n\n## Use cases and example usage\n- You have multiple remotes registered on a local git repository (or more)\nand want to push the changes to all or some of the remotes fast and efficiently.\n\n```bash\n# Use --push-only or -po\n\n$ git pp --push-only # this pushes to all remotes of the current git repository, does not stages or commits\n$ git pp --push-only --remote [one or more remotes] # only pushes to the specified remotes\n$ git pp -po --timeout 10 # terminates pushing to one remotes if it takes more than 10 seconds\n$ git pp -po -b dev ~/my-proj ~/my-proj2 # pushes the dev branch to all remotes in ~/my-proj and ~/my-proj2 repository\n```\n\n- You\'re tired of using `git add --all && git commit` every time you make a little change\nand want to automate this across one or more repositories.\n\n```bash\n$ git pp # stages all files in the current git repository and commits with a timestamp as the commit message\n$ git pp -m \'Initial commit\' # custom commit message\n$ git pp --no-status # don\'t show git status and git add outputs\n```\n\nAnd you can do both of the above (auto-commit and push) with `--push`:\n```bash\n# Use --push or -p\n\n$ git pp --push # stages, commits and pushes to all remotes.\n$ git pp --push --remote [one or more remotes]\n$ git pp -p --timeout 10\n$ git pp -p -b dev ~/my-proj ~/my-proj2\n```\n\n## Installation\n\nFirst make sure the `git` executable is installed and in your `$PATH`.\n\nNote that non-UNIX systems are not officially supported.\n\n### pipx\n\nThis is the recommended installation method.\n\n```\n$ pipx install git-pp\n```\n\n### [pip](https://pypi.org/project/git-pp/)\n```\n$ pip install git-pp\n```\n\n### [AUR](https://aur.archlinux.org/packages/python-git-pp)\nFor Archlinux.\n```\n$ yay -S python-git-pp\n```\n\n\n## Usage\n\nYou can either invoke this tool with `git-pp` or `git pp`,\n`--help` is unsupported when using the latter.\n\n```\n$ git pp -h\nusage: git pp [-h] [-m COMMIT_MESSAGE] [-v] [-so] [-p] [-po] [-r REMOTE [REMOTE ...]] [-b BRANCH] [-f] [-t TIMEOUT] [DIRS ...]\n\nGit utility for auto-committing and concurrent pushing\n\npositional arguments:\n  DIRS                  Dirs to operate on (default: [\'.\'])\n\noptions:\n  -h, --help            show this help message and exit\n  -m COMMIT_MESSAGE, --commit-message COMMIT_MESSAGE\n                        commit message (default: None)\n  -v, --version         show program\'s version number and exit\n  -so, --status-only    Prints status only (default: False)\n  -p, --push            Push to all remotes (default: False)\n  -po, --push-only      Push to all remotes, without pre_pull (default: False)\n  -r REMOTE [REMOTE ...], --remote REMOTE [REMOTE ...]\n                        Remote name (default: None)\n  -b BRANCH, --branch BRANCH\n                        Branch name (default: None)\n  -f, --force           Force push (default: False)\n  -t TIMEOUT, --timeout TIMEOUT\n                        Timeout for a single push (default: None)\n```\n\n## Develop\n```\n$ git clone https://github.com/tddschn/git-pp.git\n$ cd git-pp\n$ poetry install\n```',
    'author': 'Xinyuan Chen',
    'author_email': '45612704+tddschn@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/tddschn/git-pp',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
